<?php

namespace Drupal\content_reporting_charts\Controller;

use Drupal\node\Entity\Node;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller class for Content Reporting Charts.
 */
class ContentReportingChartsController extends ControllerBase {
  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  public function __construct(Connection $database) {
    $this->database = $database;
  }

  /**
   * Factory method for creating an instance of the controller.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container from which the database service is retrieved.
   *
   * @return static
   *   An instance of the `ContentReportingChartsController` class.
   */
  public static function create(ContainerInterface $container) {
    return new static(
          $container->get('database'),
      );
  }
 /**
   * Retrieves page views data from the 'content_reporting_interactions' table.
   *
   * @return array
   *   An associative array containing:
   *     - 'users': The list of page titles.
   *     - 'time_spent': The corresponding view counts for each page.
   */
  protected function getUserViewsData() {
    $data = [];
    
    // Execute your query to get the data.
    $results = $this->database->select('content_reporting_interactions', 't')
        ->fields('t', ['uid', 'duration'])
        ->execute()
        ->fetchAll();
    
    // Initialize counters for anonymous and authenticated users.
    $userData = [
        'Anonymous' => ['total_duration' => 0, 'count' => 0],
        'Authenticated' => ['total_duration' => 0, 'count' => 0],
    ];
    
    foreach ($results as $result) {
        $duration = $result->duration ?? 0;

        // Check if the user exists in the users table.
        $userExists = $this->database->select('users_field_data', 'u')
            ->fields('u', ['uid'])
            ->condition('uid', $result->uid)
            ->execute()
            ->fetchField();

        if ($userExists) { // User exists in the database.
            $userData['Authenticated']['total_duration'] += $duration;
            $userData['Authenticated']['count']++;
        } else { // User does not exist, so they are considered anonymous.
            $userData['Anonymous']['total_duration'] += $duration;
            $userData['Anonymous']['count']++;
        }
    }

    // Calculate the average duration for each category.
    foreach ($userData as $type => $data) {
        $userData[$type]['avg_duration'] = $data['count'] > 0 ? round($data['total_duration'] / $data['count']) : 0;
    }

    return $userData;
  }
   /**
   * Retrieves page views data from the 'content_reporting_interactions' table.
   *
   * @return array
   *   An associative array containing:
   *     - 'users': The list of page titles.
   *     - 'time_spent': The corresponding view counts for each page.
   */
  protected function getAverageTimeSpentPerPage() {
    $results = $this->database->select('content_reporting_interactions', 't')
    ->fields('t', ['nid', 'duration'])
    ->execute()
    ->fetchAll();

$data = [];

foreach ($results as $result) {
    if (!isset($data[$result->nid])) {
        $data[$result->nid] = ['total_duration' => 0, 'count' => 0];
    }
    $data[$result->nid]['total_duration'] += $result->duration;
    $data[$result->nid]['count']++;
}

foreach ($data as $nid => $values) {
    $data[$nid]['average_duration'] = round($values['total_duration'] / $values['count']);
    $data[$nid]['nid'] = $nid;
}

return $data;
}
  /**
   * Retrieves page views data from the 'content_reporting_reports' table.
   *
   * @return array
   *   An associative array containing:
   *     - 'titles': The list of page titles.
   *     - 'views': The corresponding view counts for each page.
   *     - 'gdpr_consent': The GDPR consent counts for each page.
   */
  protected function getPageViewsData() {
    $connection = $this->database;
    $query = $connection->select('content_reporting_reports', 'r')
      ->fields('r', ['title', 'views', 'gdpr_consent'])
    // Sort by page title.
      ->orderBy('title', 'ASC')
      ->execute()
      ->fetchAllAssoc('title');

    $data = [
      'titles' => [],
      'views' => [],
      'gdpr_consent' => [],
    ];

    foreach ($query as $row) {
      $data['titles'][] = $row->title;
      $data['views'][] = (int) $row->views;
      $data['gdpr_consent'][] = $row->gdpr_consent ? (int) $row->gdpr_consent : 0;
    }

    return $data;
  }

  /**
   * Builds and returns the page views dashboard chart.
   *
   * @return array
   *   A renderable array containing the chart and its configurations.
   */
  public function dashboard() {
    $data = $this->getPageViewsData();
    $charts = [];

    // Render the chart element.
    $charts['page_perview'] = [
      '#type' => 'chart',
      '#id' => 'content_reporting_page_views_chart',
      '#title' => $this->t('Views per Page'),
      '#chart_id' => 'content_reporting_page_views_chart',
    // Use a column chart to show views.
      '#chart_type' => 'column',
    ];

    // Set the series data.
    $charts['page_perview']['series'] = [
      '#type' => 'chart_data',
      '#title' => $this->t('Views'),
      '#data' => array_map(
              function ($title, $views) {
                  return ['name' => $title, 'y' => $views];
              },
              $data['titles'],
              $data['views']
      ),
    ];

    // Set the x-axis labels.
    $charts['page_perview']['x_axis'] = [
      '#type' => 'chart_xaxis',
      '#labels' => $data['titles'],
    ];

    // Set the y-axis label.
    $charts['page_perview']['y_axis'] = [
      '#type' => 'chart_yaxis',
      '#title' => $this->t('Counts'),
    ];


    $dataAVG = $this->getUserViewsData();

    $charts['time_spent_per_user'] = [
        '#type' => 'chart',
        '#id' => 'content_reporting_time_spent_per_user_chart',
        '#title' => $this->t('Average Time Spent'),
        '#chart_type' => 'bar', // Use a line chart.
    ];
    
    // Set the series data for anonymous and authenticated users.
    $charts['time_spent_per_user']['series'] = [
        '#type' => 'chart_data',
        '#title' => $this->t('Average Time Spent'),
        '#data' => [
            ['name' => 'Anonymous', 'y' => $dataAVG['Anonymous']['avg_duration']],
            ['name' => 'Authenticated', 'y' => $dataAVG['Authenticated']['avg_duration']],
        ],
    ];
    
    // Configure the x-axis labels (only two categories).
    $charts['time_spent_per_user']['x_axis'] = [
        '#type' => 'chart_xaxis',
        '#labels' => ['Anonymous', 'Authenticated'],
    ];
    
    // Set the y-axis label.
    $charts['time_spent_per_user']['y_axis'] = [
        '#type' => 'chart_yaxis',
        '#title' => $this->t('Average Duration (seconds)'),
    ];

    $dataAVG2 = $this->getAverageTimeSpentPerPage();

    $charts['average_time_spent_per_page'] = [
        '#type' => 'chart',
        '#id' => 'content_reporting_average_time_spent_per_page_chart',
        '#title' => $this->t('Average Time Spent per Page'),
        '#chart_type' => 'pie',
    ];
    
    // Set the series data for average duration.
    $charts['average_time_spent_per_page']['series'] = [
        '#type' => 'chart_data',
        '#title' => $this->t('Average Time Spent'),
        '#data' => array_values(array_map(
          function ($result) {
              return [
                  'name' => $this->getNodeTitle($result['nid']),
                  'y' => $result['average_duration'],
              ];
          },
          $dataAVG2
      )),
    ];
    
    return [
      '#theme' => 'content_reporting_charts_dashboard',
      '#attached' => [
        'library' => [
          'content_reporting/content_reporting.styles', // Attach library.
        ],
      ],
      '#charts' => $charts,
    ];
  }
/**
 * Get node title by nid.
 *
 * @param int $nid
 *   ID of node.
 *
 * @return string|null
 *   Node title or string.
 */
protected function getNodeTitle($nid) {
  // Load node.
  if(!$nid) {
    return 'No Title';
  }
  $node = Node::load($nid);

  return $node ? $node->getTitle() : 'No Title';
}
}
