<?php

namespace Drupal\content_reporting\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Session\SessionInterface;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Provides methods for tracking content views and interactions.
 */
class TrackingController extends ControllerBase {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The session manager service.
   * 
   * @var \Symfony\Component\HttpFoundation\Session\SessionInterface
   */
  protected $session;

  /**
   * Constructs a TrackingController object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Symfony\Component\HttpFoundation\Session\SessionInterface $session
   *   The session manager service.
   */
  public function __construct(Connection $database, AccountProxyInterface $current_user, EntityTypeManagerInterface $entity_type_manager, SessionInterface $session) {
    $this->database = $database;
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->session = $session;
  }

  /**
   * Creates an instance of the TrackingController.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   *
   * @return static
   *   The controller instance.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('current_user'),
      $container->get('entity_type.manager'),
      $container->get('session')
    );
  }

  /**
   * Tracks content views and GDPR consent.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response indicating the status of the operation.
   */
  public function trackContent(Request $request) {
    // Validate the CSRF token.
    $csrf_token = $request->headers->get('X-CSRF-Token');
    if (!$csrf_token || !$this->isValidCsrfToken($csrf_token)) {
      throw new AccessDeniedHttpException('Invalid CSRF token.');
    }

    // Retrieve the request body.
    $data = json_decode($request->getContent(), TRUE);
    if (isset($data['nid'])) {
      $node_id = intval($data['nid']);
      $node_storage = $this->entityTypeManager->getStorage('node');
      $node = $node_storage->load($node_id);

      if ($node) {
        $node_title = $node->getTitle();
        $gdpr_consent = isset($data['gdpr_consent']) && $data['gdpr_consent'] == TRUE ? $data['gdpr_consent'] : NULL;

        $this->trackNode($node_id, $node_title, $gdpr_consent);

        return new JsonResponse(['status' => 'success']);
      }
      return new JsonResponse(['status' => 'error', 'message' => 'Node not found'], Response::HTTP_BAD_REQUEST);
    }

    return new JsonResponse(['status' => 'error', 'message' => 'Missing node ID'], Response::HTTP_BAD_REQUEST);
  }

  /**
   * Tracks the node view and GDPR consent.
   *
   * @param int $node_id
   *   The node ID.
   * @param string $node_title
   *   The node title.
   * @param int|null $gdpr_consent
   *   The GDPR consent value, or NULL if not applicable.
   */
  protected function trackNode($node_id, $node_title, $gdpr_consent) {
    $anonymous_user_id = $this->generateAnonymousUserIdSession();
    $uid = $this->currentUser->isAuthenticated() ? $this->currentUser->id() : $anonymous_user_id;

    // More rigorous GDPR consent check.
    $consent = is_null($gdpr_consent) ? NULL : (int) $gdpr_consent;

    // Insert or update the record in the database.
    $query = $this->database->merge('content_reporting_reports')
      ->key(['nid' => $node_id, 'uid' => $uid])
      ->fields([
        'title' => $node_title,
        'views' => 1,
        'report_date' => \Drupal::time()->getRequestTime(),
      ])
      ->expression('views', 'COALESCE(views, 0) + 1');

    // Update the GDPR consent count if consent was given (gdpr_consent == 1)
    if ($consent == TRUE) {
      $query->expression('gdpr_consent', 'COALESCE(gdpr_consent, 0) + 1');
    }
    // Execute the query for the main record.
    $query->execute();
  }

  /**
   * Tracks user interactions with content.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response indicating the status of the operation.
   */
  public function trackInteraction(Request $request) {
    // Validate the CSRF token.
    $csrf_token = $request->headers->get('X-CSRF-Token');
    if (!$csrf_token || !$this->isValidCsrfToken($csrf_token)) {
      throw new AccessDeniedHttpException('Invalid CSRF token.');
    }

    // Retrieve the request body.
    $data = json_decode($request->getContent(), TRUE);
    $anonymous_user_id = $this->generateAnonymousUserIdSession();
    if (isset($data['nid']) && isset($data['interaction_type'])) {
      $node_id = intval($data['nid']);
      $uid = $this->currentUser->isAuthenticated() ? $this->currentUser->id() : $anonymous_user_id;
      $element = $data['element'] ?? NULL;
      $duration = $data['duration'] ?? NULL;
      $interaction_type = $data['interaction_type'];

      // Save the interaction in the database.
      $this->database->insert('content_reporting_interactions')
        ->fields([
          'nid' => $node_id,
          'uid' => $uid,
          'interaction_type' => $interaction_type,
          'element' => $element,
          'duration' => $duration,
          'timestamp' => \Drupal::time()->getRequestTime(),
        ])
        ->execute();

      return new JsonResponse(['status' => 'success']);
    }

    return new JsonResponse(['status' => 'error', 'message' => 'Missing data'], Response::HTTP_BAD_REQUEST);
  }

  /**
   * Validates the CSRF token.
   *
   * @param string $token
   *   The CSRF token to validate.
   *
   * @return bool
   *   TRUE if the token is valid, FALSE otherwise.
   */
  protected function isValidCsrfToken($token) {
    // Dummy CSRF token validation (replace with real implementation)
    return TRUE;
    // In a real context, you can use the csrf_token service to validate:
    // return \Drupal::csrfToken()->validate($token);
  }
/**
   * Generates a unique anonymous user ID based on the session.
   *
   *
   * @return string
   *   A unique identifier for the anonymous user.
   */
  protected function generateAnonymousUserIdSession() {
    // Check if the anonymous user already has a session-stored ID.
    if (!$this->session->has('anonymous_user_id')) {
      // Generate a unique numeric ID for the anonymous user.
      $anonymous_user_id = 10000 + mt_rand(0, 9999); // Ensure the ID is numeric and unique.
      
      // Store the generated ID in the session.
      $this->session->set('anonymous_user_id', $anonymous_user_id);
  }

  // Return the stored anonymous user ID from the session.
  return (int) $this->session->get('anonymous_user_id'); // Cast to int to ensure it's numeric.
  }
}
